/******************************************************************************
 * $Id: src/display.c, v0.01 6:16 02.01.2011 TiN Exp $
 *
 * Copyright (c) 2010, xDevs.com ARM Microcontroller Software Support
 *
 * LO-13232A-02 ST7588T based LCD driver
 *
 * Project maintainers:
 *  o Tsemenko Ilya  (ilya@xdevs.com)
 *  o Alex Gurin     (warder@xdevs.com)
 */

#include <stdio.h>
#include <stdarg.h>
#include "xprintf.h"
#include "lcd.h"
#include "display.h"


extern hc1_lcd_cell_value_t  ascii_lut[];
extern hc1_lcd_icon_value_t  icon_lut[];


#define write_char_to_cell(ch, x, y)                            \
    do {                                                        \
        uint8_t ci;                                             \
                                                                \
        if ((x >= 22) || (y >= 4))                              \
            return -1;                                          \
                                                                \
        /* compute cell index */                                \
        ci = (y * 22) + x;                                      \
                                                                \
        hc1_lcd_update_cell_value(ci, &ascii_lut[ch - 0x20]);   \
    } while (0)


/**
  * @param ch
  *         The character to print
  * @param x
  *         The X coordinate of the cartesian coordinate system
  * @param y
  *         The Y coordinate of the cartesian coordinate system
  * This function writes to specified coordinates a character,
  * on success it returns a character itself, otherwise -1 on failures.
  */
int hc1_putc(uint8_t ch, uint8_t x, uint8_t y)
{
    write_char_to_cell(ch, x, y);
    hc1_lcd_update_cell_value(0, NULL);
    return ch;
}


int hc1_printf(uint8_t x, uint8_t y, uint8_t *fmt, ...)
{
    int length = 0;
    uint8_t buf[128], *s = buf;
    va_list vl;
    uint8_t xi, yi;
    
    va_start(vl, fmt);
    xvsnprintf(buf, sizeof(buf), fmt, vl);
    va_end(vl);
    
    for (yi = y; yi < 4; yi++) {
        for (xi = x; (xi < 22) && (*s); xi++) {
            write_char_to_cell(*s++, xi, yi);
        }
    }
    hc1_lcd_update_cell_value(0, NULL);
    
    return length;
}


